@@ -1,26 +1,34 @@
 import asyncio
 from asyncio import StreamReader, StreamWriter
 from copy import copy
+from dataclasses import dataclass
 from enum import Enum
 from os import getenv
-from typing import Union, Dict, Any, Optional, List
+from pathlib import Path
+from subprocess import run
+from tempfile import TemporaryDirectory, NamedTemporaryFile
+from typing import Union, Dict, Any, Optional, List, Tuple
 
 import aioredis
 import pytest
+from cryptography.hazmat.primitives import serialization
+from cryptography.hazmat.primitives.asymmetric import rsa
 from python_on_whales import docker, Image, Container
 
-DNS_SUFFIX = '.dlemel8.xyz'
+DNS_CLIENT_SUFFIX = '.dlemel8.xyz'
+TLS_SERVER_NAME = 'server.tunneler'
 
 
 class TestPorts(Enum):
-    BACKEND_PORT = 8080
+    BACKEND_PORT = 8089
     UNTUNNELER_PORT = 8899
-    TUNNELER_PORT = 8888
+    TUNNELER_PORT = 8889
 
 
 class TunnelerType(Enum):
     TCP = 'tcp'
     DNS = 'dns'
+    TLS = 'tls'
 
 
 class TunneledType(Enum):
@@ -114,7 +122,8 @@ def run_tunneler_container(image: Image,
                            tunneled_type: TunneledType,
                            local_port: Union[TestPorts, int],
                            remote_port: TestPorts,
-                           extra_env_vars: Optional[Dict[str, Any]] = None) -> Container:
+                           extra_env_vars: Optional[Dict[str, Any]] = None,
+                           volumes: Optional[List[Tuple[str, str]]] = None) -> Container:
     if isinstance(local_port, TestPorts):
         local_port_value = local_port.value
     elif isinstance(local_port, int):
@@ -138,6 +147,7 @@ def run_tunneler_container(image: Image,
             **extra_env_vars,
         },
         networks=['host'],
+        volumes=volumes or [],
     )
 
 
@@ -164,7 +174,7 @@ def tcp_over_dns_server(server_image: Image) -> Container:
                                        TestPorts.BACKEND_PORT,
                                        extra_env_vars={'READ_TIMEOUT_IN_MILLISECONDS': 100,
                                                        'IDLE_CLIENT_TIMEOUT_IN_MILLISECONDS': 300_000,
-                                                       'CLIENT_SUFFIX': DNS_SUFFIX})
+                                                       'CLIENT_SUFFIX': DNS_CLIENT_SUFFIX})
     yield container
     print_log_and_delete_container(container)
 
@@ -179,7 +189,7 @@ def udp_over_dns_server(server_image: Image) -> Container:
                                        TestPorts.BACKEND_PORT,
                                        extra_env_vars={'READ_TIMEOUT_IN_MILLISECONDS': 100,
                                                        'IDLE_CLIENT_TIMEOUT_IN_MILLISECONDS': 300_000,
-                                                       'CLIENT_SUFFIX': DNS_SUFFIX})
+                                                       'CLIENT_SUFFIX': DNS_CLIENT_SUFFIX})
     yield container
     print_log_and_delete_container(container)
 
@@ -208,6 +218,81 @@ def udp_over_tcp_server(server_image: Image) -> Container:
     print_log_and_delete_container(container)
 
 
+@pytest.fixture(scope='session')
+def ca_private_ca() -> str:
+    with NamedTemporaryFile() as key_file:
+        private_key = rsa.generate_private_key(public_exponent=65537, key_size=2048)
+        pem = private_key.private_bytes(
+            encoding=serialization.Encoding.PEM,
+            format=serialization.PrivateFormat.TraditionalOpenSSL,
+            encryption_algorithm=serialization.NoEncryption(),
+        )
+        key_file.write(pem)
+        key_file.flush()
+        yield key_file.name
+
+
+@dataclass
+class PkiPemPaths:
+    ca_certificate: Path
+    server_key: Path
+    server_certificate: Path
+    client_key: Path
+    client_certificate: Path
+
+
+@pytest.fixture(scope='session')
+def pki(ca_private_ca: str) -> PkiPemPaths:
+    with TemporaryDirectory() as pki_dir:
+        run(f'bash pki.sh -k {ca_private_ca} -t {pki_dir} ca server client', shell=True, check=True)
+        tmp_dir_path = Path(pki_dir)
+        yield PkiPemPaths(
+            tmp_dir_path.joinpath('ca.crt'),
+            tmp_dir_path.joinpath('server.key'),
+            tmp_dir_path.joinpath('server.crt'),
+            tmp_dir_path.joinpath('client.key'),
+            tmp_dir_path.joinpath('client.crt'),
+        )
+
+
+@pytest.fixture
+def tcp_over_tls_server(server_image: Image, pki: PkiPemPaths) -> Container:
+    container = run_tunneler_container(server_image,
+                                       'test_server',
+                                       TunnelerType.TLS,
+                                       TunneledType.TCP,
+                                       TestPorts.UNTUNNELER_PORT,
+                                       TestPorts.BACKEND_PORT,
+                                       extra_env_vars={'CA_CERT': '/ca_certificate',
+                                                       'KEY': '/server_key',
+                                                       'CERT': '/server_certificate',
+                                                       'SERVER_HOSTNAME': TLS_SERVER_NAME},
+                                       volumes=[(str(pki.ca_certificate), '/ca_certificate'),
+                                                (str(pki.server_key), '/server_key'),
+                                                (str(pki.server_certificate), '/server_certificate')])
+    yield container
+    print_log_and_delete_container(container)
+
+
+@pytest.fixture
+def udp_over_tls_server(server_image: Image, pki: PkiPemPaths) -> Container:
+    container = run_tunneler_container(server_image,
+                                       'test_server',
+                                       TunnelerType.TLS,
+                                       TunneledType.UDP,
+                                       TestPorts.UNTUNNELER_PORT,
+                                       TestPorts.BACKEND_PORT,
+                                       extra_env_vars={'CA_CERT': '/ca_certificate',
+                                                       'KEY': '/server_key',
+                                                       'CERT': '/server_certificate',
+                                                       'SERVER_HOSTNAME': TLS_SERVER_NAME},
+                                       volumes=[(str(pki.ca_certificate), '/ca_certificate'),
+                                                (str(pki.server_key), '/server_key'),
+                                                (str(pki.server_certificate), '/server_certificate')])
+    yield container
+    print_log_and_delete_container(container)
+
+
 @pytest.fixture(scope='session')
 def client_image() -> Image:
     image = build_tunneler_image('client', 'test_client')
@@ -225,7 +310,7 @@ def tcp_over_dns_client(client_image: Image) -> Container:
                                        TestPorts.UNTUNNELER_PORT,
                                        extra_env_vars={'READ_TIMEOUT_IN_MILLISECONDS': 100,
                                                        'IDLE_CLIENT_TIMEOUT_IN_MILLISECONDS': 30000,
-                                                       'CLIENT_SUFFIX': DNS_SUFFIX})
+                                                       'CLIENT_SUFFIX': DNS_CLIENT_SUFFIX})
     yield container
     print_log_and_delete_container(container)
 
@@ -240,7 +325,7 @@ def udp_over_dns_client(client_image: Image) -> Container:
                                        TestPorts.UNTUNNELER_PORT,
                                        extra_env_vars={'READ_TIMEOUT_IN_MILLISECONDS': 100,
                                                        'IDLE_CLIENT_TIMEOUT_IN_MILLISECONDS': 30000,
-                                                       'CLIENT_SUFFIX': DNS_SUFFIX})
+                                                       'CLIENT_SUFFIX': DNS_CLIENT_SUFFIX})
     yield container
     print_log_and_delete_container(container)
 
@@ -269,6 +354,44 @@ def udp_over_tcp_client(client_image: Image) -> Container:
     print_log_and_delete_container(container)
 
 
+@pytest.fixture
+def tcp_over_tls_client(client_image: Image, pki: PkiPemPaths) -> Container:
+    container = run_tunneler_container(client_image,
+                                       'test_client',
+                                       TunnelerType.TLS,
+                                       TunneledType.TCP,
+                                       TestPorts.TUNNELER_PORT,
+                                       TestPorts.UNTUNNELER_PORT,
+                                       extra_env_vars={'CA_CERT': '/ca_certificate',
+                                                       'KEY': '/client_key',
+                                                       'CERT': '/client_certificate',
+                                                       'SERVER_HOSTNAME': TLS_SERVER_NAME},
+                                       volumes=[(str(pki.ca_certificate), '/ca_certificate'),
+                                                (str(pki.client_key), '/client_key'),
+                                                (str(pki.client_certificate), '/client_certificate')])
+    yield container
+    print_log_and_delete_container(container)
+
+
+@pytest.fixture
+def udp_over_tls_client(client_image: Image, pki: PkiPemPaths) -> Container:
+    container = run_tunneler_container(client_image,
+                                       'test_client',
+                                       TunnelerType.TLS,
+                                       TunneledType.UDP,
+                                       TestPorts.TUNNELER_PORT,
+                                       TestPorts.UNTUNNELER_PORT,
+                                       extra_env_vars={'CA_CERT': '/ca_certificate',
+                                                       'KEY': '/client_key',
+                                                       'CERT': '/client_certificate',
+                                                       'SERVER_HOSTNAME': TLS_SERVER_NAME},
+                                       volumes=[(str(pki.ca_certificate), '/ca_certificate'),
+                                                (str(pki.client_key), '/client_key'),
+                                                (str(pki.client_certificate), '/client_certificate')])
+    yield container
+    print_log_and_delete_container(container)
+
+
 async def run_test_tcp_single_client_single_short_echo() -> None:
     message_to_send = 'bla'
     reader, writer = await asyncio.open_connection('127.0.0.1', TestPorts.TUNNELER_PORT.value)
