@@ -13,8 +13,7 @@ use async_trait::async_trait;
 use mockall::automock;
 use tokio::io::{duplex, split};
 use tokio::net::UdpSocket;
-use tokio::time::timeout;
-use tokio::time::{Duration, Instant};
+use tokio::time::{timeout, Duration, Instant};
 use trust_dns_client::op::{Header, MessageType, OpCode};
 use trust_dns_client::proto::rr::rdata::TXT;
 use trust_dns_client::proto::rr::{Name, RData, Record, RecordType};
@@ -22,13 +21,13 @@ use trust_dns_server::authority::{MessageRequest, MessageResponse, MessageRespon
 use trust_dns_server::server::{Request, RequestHandler, ResponseHandler};
 use trust_dns_server::ServerFuture;
 
+use common::cache::{StreamCreator, StreamsCache};
 use common::dns::{
     AppendSuffixDecoder, ClientId, ClientIdSuffixDecoder, Decoder, Encoder, HexDecoder, HexEncoder,
     CLIENT_ID_SIZE_IN_BYTES,
 };
 use common::io::Stream;
 
-use crate::io::{StreamCreator, StreamsCache};
 use crate::tunnel::Untunneler;
 
 pub(crate) struct DnsUntunneler {
@@ -59,7 +58,7 @@ impl DnsUntunneler {
 #[async_trait(? Send)]
 impl Untunneler for DnsUntunneler {
     async fn untunnel(&mut self, new_clients: Sender<Stream>) -> Result<(), Box<dyn Error>> {
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             move || {
                 let (local, remote) = duplex(4096);
 
@@ -70,7 +69,6 @@ impl Untunneler for DnsUntunneler {
                 Ok(Stream::new(local_reader, local_writer))
             },
             self.idle_client_timeout,
-            Duration::from_secs(60),
         );
 
         let udp_socket = UdpSocket::bind(self.listener_address).await?;
@@ -353,10 +351,9 @@ mod tests {
         };
 
         let handler_mock = MockDnsResponseHandler::new();
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || Ok(Stream::new(Builder::new().build(), Builder::new().build())),
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let decoder_mock = MockDecoder::new();
         let encoder_mock = MockEncoder::new();
@@ -388,10 +385,9 @@ mod tests {
         };
 
         let handler_mock = MockDnsResponseHandler::new();
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || Ok(Stream::new(Builder::new().build(), Builder::new().build())),
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let decoder_mock = MockDecoder::new();
         let encoder_mock = MockEncoder::new();
@@ -421,10 +417,9 @@ mod tests {
         };
 
         let handler_mock = MockDnsResponseHandler::new();
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || Ok(Stream::new(Builder::new().build(), Builder::new().build())),
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let decoder_mock = MockDecoder::new();
         let encoder_mock = MockEncoder::new();
@@ -456,10 +451,9 @@ mod tests {
         };
 
         let handler_mock = MockDnsResponseHandler::new();
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || Ok(Stream::new(Builder::new().build(), Builder::new().build())),
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let mut decoder_mock = MockDecoder::new();
         decoder_mock
@@ -494,10 +488,9 @@ mod tests {
         };
 
         let handler_mock = MockDnsResponseHandler::new();
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || Ok(Stream::new(Builder::new().build(), Builder::new().build())),
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let mut decoder_mock = MockDecoder::new();
         decoder_mock
@@ -535,10 +528,9 @@ mod tests {
         };
 
         let handler_mock = MockDnsResponseHandler::new();
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || Err(String::from("bla").into()),
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let mut decoder_mock = MockDecoder::new();
         decoder_mock
@@ -576,7 +568,7 @@ mod tests {
         };
 
         let handler_mock = MockDnsResponseHandler::new();
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || {
                 Ok(Stream::new(
                     Builder::new().build(),
@@ -586,7 +578,6 @@ mod tests {
                 ))
             },
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let mut decoder_mock = MockDecoder::new();
         decoder_mock
@@ -624,7 +615,7 @@ mod tests {
         };
 
         let handler_mock = MockDnsResponseHandler::new();
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || {
                 Ok(Stream::new(
                     Builder::new()
@@ -634,7 +625,6 @@ mod tests {
                 ))
             },
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let mut decoder_mock = MockDecoder::new();
         decoder_mock
@@ -672,7 +662,7 @@ mod tests {
         };
 
         let handler_mock = MockDnsResponseHandler::new();
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || {
                 Ok(Stream::new(
                     Builder::new().read(b"bli").build(),
@@ -680,7 +670,6 @@ mod tests {
                 ))
             },
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let mut decoder_mock = MockDecoder::new();
         decoder_mock
@@ -724,7 +713,7 @@ mod tests {
         handler_mock
             .expect_send_response()
             .returning(|_| Err(io::Error::new(ErrorKind::Other, "oh no!")));
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || {
                 Ok(Stream::new(
                     Builder::new().read(b"bli").build(),
@@ -732,7 +721,6 @@ mod tests {
                 ))
             },
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let mut decoder_mock = MockDecoder::new();
         decoder_mock
@@ -774,7 +762,7 @@ mod tests {
 
         let mut handler_mock = MockDnsResponseHandler::new();
         handler_mock.expect_send_response().returning(|_| Ok(()));
-        let cache = StreamsCache::new(
+        let cache = StreamsCache::with_default_cleanup_duration(
             || {
                 Ok(Stream::new(
                     Builder::new().read(b"bli").build(),
@@ -782,7 +770,6 @@ mod tests {
                 ))
             },
             Duration::from_secs(3 * 60),
-            Duration::from_secs(60),
         );
         let mut decoder_mock = MockDecoder::new();
         decoder_mock
